/* globals Ext, dSS */

Ext.namespace('dSS', 'dSS.data', 'dSS11');

dSS.data.ModulatorStore = Ext.extend(Ext.data.Store, {
    constructor: function () {  // config
        // create a record constructor for zone records
        var modulatorRecord = Ext.data.Record.create([
            {
                name: 'busId',
                mapping: 'busID'
            },
            {
                name: 'deviceType'
            },
            {
                name: 'dsuid',
                mapping: 'dSUID'
            },
            {
                name: 'dsid'
            },
            {
                name: 'DisplayID'
            },
            {
                name: 'busMemberType'
            },
            {
                name: 'hwName'
            },
            {
                name: 'hwVersionString'
            },
            {
                name: 'swVersion'
            },
            {
                name: 'armSwVersion'
            },
            {
                name: 'dspSwVersion'
            },
            {
                name: 'isUpToDate'
            },
            {
                name: 'apiVersion'
            },
            {
                name: 'isPresent'
            },
            {
                name: 'hasMetering'
            },
            {
                name: 'hasBlinking'
            },
            {
                name: 'hasDevices'
            },
            {
                name: 'name'
            },
            {
                name: 'consumption'
            },
            {
                name: 'production'
            },
            {
                name: 'measureDate'
            },
            {
                name: 'isValid'
            },
            {
                name: 'numDevices',
                defaultValue: 0
            },
            {
                name: 'ledCode',
                defaultValue: -1
            },
            {
                name: 'ledStatus'
            },
            {
                name: 'ignoreActionsFromNewDevices'
            },
            {
                name: 'VdcConfigURL'
            }
        ]);

        // a json reader to read the zone data
        var modulatorReader = new Ext.data.JsonReader({
            root: 'result.circuits',
            idProperty: 'dSUID'
        }, modulatorRecord);

        this.addEvents('loadcomplete');

        Ext.apply(this, {
            reader: modulatorReader,
            url: '/json/apartment/getCircuits'
        });
        dSS.data.ModulatorStore.superclass.constructor.call(this, arguments);

        this.on('exception', function () {
            Ext.Msg.alert(dSS._('Error'), dSS._('Could not load data from dSS!'));
            dSS.MessageBus.fireEvent('structureloaded');
        }, this);
        this.on('load', function (store, records) { // , options
            Ext.each(records, function (record) {
                // remove anything than dsm's and virtual device containers
                if (
                    record.get('busMemberType') !== dSS.config.BusMemberTypes.dSM11 &&
                    record.get('busMemberType') !== dSS.config.BusMemberTypes.dSM12 &&
                    record.get('busMemberType') !== dSS.config.BusMemberTypes.dSM20 &&
                    record.get('busMemberType') !== dSS.config.BusMemberTypes.dSM25 &&
                    record.get('busMemberType') !== dSS.config.BusMemberTypes.VDC &&
                    record.get('busMemberType') !== dSS.config.BusMemberTypes.backendVdc
                ) {
                    store.remove(record);
                    return;
                }
                record.set('hwVersion', dSS.util.decode32(record.get('hwVersion')));
                record.set('armSwVersion', dSS.util.decode32(record.get('armSwVersion')));
                record.set('dspSwVersion', dSS.util.decode32(record.get('dspSwVersion')));
                record.set('apiVersion', dSS.util.decode16V(record.get('apiVersion')));
            });

            Ext.Ajax.request({
                url: '/json/metering/getLatest',
                disableCaching: true,
                method: 'GET',
                scope: this,
                params: {
                    type: 'consumption',
                    from: '.meters(all)',
                    withProduction: true
                },
                success: function (result) {    // , request
                    try {
                        var jsonData = Ext.util.JSON.decode(result.responseText);
                        if (jsonData.ok) {
                            var modified = false;
                            Ext.each(jsonData.result.values, function (val) {
                                var index = store.find('dsuid', new RegExp('^' + val.dSUID + '$'));
                                if (index > -1) {
                                    var rec = store.getAt(index);
                                    rec.set('consumption', val.consumption);
                                    rec.set('production', val.production);
                                    rec.set('measureDate', val.date);
                                    modified = true;
                                }
                            });
                            if (modified) {
                                store.commitChanges();
                            }

                        } else {
                            Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not get metering data from server'));
                        }
                    } catch (err) {
                        Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not get metering data from server'));
                    }
                    this.loadVisibilityMatrix(store);
                },
                failure: function () {  // result, request
                    Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not get metering data from server'));
                    this.loadVisibilityMatrix(store);
                }
            });
            dSS.util.getControllerStatuses(store);
        },
        this);
    },

    loadVisibilityMatrix: function (store) {
        Ext.Ajax.request({
            url: '/json/apartment/getModelFeatures2',
            disableCaching: true,
            method: 'GET',
            scope: this,
            success: function (result) {    // , request
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._('Error'), dSS._('Failure when requesting model features from dSS'));
                        this.getARMImageVersion(store);
                        return;
                    }

                    dSS.devprops.VisibilityMatrix = json.result.modelFeatures;
                    dSS.devprops.ErrorMatrix = json.result.reference;
                } catch (err) {
                    Ext.Msg.alert(dSS._('Error'), dSS._('Could not get model features from dSS') + ': ' + err);
                }
                this.getARMImageVersion(store);
            },
            failure: function () {
                Ext.Msg.alert(dSS._('Error'), dSS._('Failure when requesting model features from dSS'));
                this.getARMImageVersion(store);
            }
        });
    },

    // have to use two calls because query behaved buggy and delivered
    // non-determenistic results
    getARMImageVersion: function (store) {
        Ext.Ajax.request({
            url: '/json/property/getString',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                path: '/system/version/dSMFirmwareImageARM'
            },
            success: function (result) {    // , request
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        if (jsonData.result.value.length) {
                            dSS.config.dSMSystemFirmwareImageVersion.ARM = jsonData.result.value;
                        }
                        this.getDSPImageVersion(store);
                    } else {
                        this.finishLoading();
                    }
                } catch (err) {
                    this.finishLoading();
                }
            },
            failure: function () {  // result, request
                this.finishLoading();
            }
        });
    },

    getDSPImageVersion: function () {   // store
        Ext.Ajax.request({
            url: '/json/property/getString',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                path: '/system/version/dSMFirmwareImageDSP'
            },
            success: function (result) {    // , request
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        if (jsonData.result.value.length) {
                            dSS.config.dSMSystemFirmwareImageVersion.DSP = jsonData.result.value;
                        }
                    }
                } catch (err) {
                    // silence error
                }
                this.finishLoading();
            },
            failure: function () {  // result, request
                this.finishLoading();
            }
        });
    },

    finishLoading: function () {    // store
        this.fireEvent('loadcomplete');
    }
});
