// parameters:
//      dSUID: device dSUID

// bank 64, offsets 0x03, 0x02 (2B) - setting shade position
// bank 64, offset 0x04 (1B) - set lamellas
// bank 3, offsets 0x3f, 0x3e (2B) - turn time adjustment

/* globals dSS, Ext */

dSS.devprops.GRKL300FinsCalibrationWizard = Ext.extend(Ext.Window, {
    constructor: function (config) {

        if (config.dSUID == null) {
            throw dSS._('Missing dSUID');
        }

        if (
            config.mtimeadjust == null ||       // DimStep length
            config.motionTimeAndDirectionCalibrationStatus == null
        ) {
            throw dSS._('GRKL300FinsCalibrationWizard: called with missing parameters');
        }

        this.loaded = {
            mtimeadjust: config.mtimeadjust,
            motionTimeAndDirectionCalibrationStatus: config.motionTimeAndDirectionCalibrationStatus,
            reverseTimeOnDirectionChange: config.reverseTimeOnDirectionChange
        };

        this.currentStep = 0;

        this.INC_DEC_VALUE = 20;
        this.SHADE_POSITION_100_PERC = 0xffff;  // shade position is in range <0..0xffff>
        this.SHADE_POSITION_50_PERC = 0x8000;   // as the real step of the device may differ, the value is not exact
        this.lastReadValue = null;              // so in order to determine the position, I'll check whether it's less than 10% from middle value and not changing any more
        this.dSUID = config.dSUID;
        this.tempTurnTime = 0;
        this.incStepCounter = 0;

        this.finsAngle = {
            OPEN: 0,
            CENTER: 1,
            CLOSED: 2
        };

        this.angle = this.finsAngle.CLOSED;

        var devStore = dSS.data.Loader.getDeviceStore();
        var record = devStore.getById(this.dSUID);
        if (record == null) {
            throw dSS._('Device not found');
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var number = devInfo[1];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        if (cls[0] !== dSS.config.deviceClass.GR) {
            throw dSS._('Unsupported device group');
        }

        if (
            !(dSS.config.deviceTypes[type] === 'KL' && number === 300) &&
            !(dSS.config.deviceTypes[type] === 'HKL' && number === 330)
        ) {
            throw dSS._('Unsupported device type');
        }

        var text1 = dSS._('Fins: no text given 1');     // "The blind will move to the required position for calibrating the turn time first [...]"
        var text2 = dSS._('Fins: no text given 2');     // "The determined turn time can now be fine-tuned. Open and close the blades [...]"
        this.buttonWidth = 80;

        this.cancelButton = new Ext.Button({
            cls: 'dsClsFinsCalibrationWizard_CancelButton',

            xtype: 'button',
            text: dSS._('Cancel'),
            width: this.buttonWidth,
            scope: this,
            handler: this.cancelWizard.createDelegate(this)
        });

        this.continueButton = new Ext.Button({
            cls: 'dsClsFinsCalibrationWizard_ContinueButton',

            text: dSS._('Continue'),
            handler: this.handleContinue.createDelegate(this),
            disabled: true,
            width: this.buttonWidth
        });

        Ext.apply(this, {
            cls: 'dsClsFinsCalibrationWizard',

            layout: 'fit',
            closable: false,
            width: dSS.gthelper.getDialogConfig('finsCalibrationWindowWidth'),
            height: 380,
            resizable: false,
            modal: true,
            title: dSS._('Calibration of fins turn time: step 1 of 2'),
            items: [
                {
                    layout: 'card',
                    ref: 'main',
                    padding: 10,
                    border: false,
                    bodyCfg: {
                        cls: 'x-window-body x-window-mc'
                    },
                    items: [
                        {
                            cls: 'dsClsFinsCalibrationWizard_Step1Panel',

                            xtype: 'panel',
                            ref: 'step1',
                            height: 350,
                            layout: 'border',
                            border: false,
                            items: [
                                {
                                    xtype: 'panel',
                                    region: 'north',
                                    height: 270,
                                    html: text1,
                                    border: false,
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    }
                                },
                                {
                                    xtype: 'panel',
                                    border: false,
                                    ref: 'action',
                                    region: 'center',
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    },
                                    layout: 'hbox',
                                    items: [
                                        {
                                            xtype: 'label',
                                            width: 120,
                                            ref: 'numsteps',
                                            text: dSS._('Number of steps') + ': 0'
                                        },
                                        {
                                            xtype: 'box',
                                            flex: 3
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step1Panel_UpButton',

                                            xtype: 'button',
                                            iconCls: 'buttonUpIcon',
                                            ref: 'incButton',
                                            disabled: true,
                                            width: 79,
                                            handler: this.handleInc.createDelegate(this)
                                        },
                                        {
                                            xtype: 'box',
                                            width: 6
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step1Panel_StopButton',

                                            xtype: 'button',
                                            ref: 'stopButton',
                                            text: dSS._('Stop'),
                                            width: 79,
                                            handler: this.handleStop.createDelegate(this)
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            cls: 'dsClsFinsCalibrationWizard_Step2Panel',

                            xtype: 'panel',
                            hidden: true,
                            layout: 'border',
                            border: false,
                            height: 350,
                            scope: this,
                            ref: 'step2',
                            items: [
                                {
                                    xtype: 'panel',
                                    region: 'north',
                                    height: 210,
                                    html: text2,
                                    border: false,
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    }
                                },
                                {
                                    xtype: 'panel',
                                    region: 'center',
                                    height: 100,
                                    border: false,
                                    ref: 'action',
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    },
                                    layout: 'absolute',
                                    items: [
                                        {
                                            xtype: 'label',
                                            text: dSS._('Fins turn time') + ':',
                                            x: 0,
                                            y: 4
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_FinsTurnTimeSpinnerField',

                                            xtype: 'spinnerfield',
                                            width: 50,
                                            ref: 'turnspinner',
                                            allowBlank: false,
                                            minValue: 0,
                                            allowDecimals: true,
                                            accelerate: false,
                                            defaultValue: 0,
                                            splitterClass: ' ',
                                            incrementValue: 0.05,
                                            decimalPrecision: 2,
                                            disabled: false,
                                            allowEmpty: false,
                                            x: 150,
                                            y: 0
                                        },
                                        {
                                            xtype: 'label',
                                            text: dSS._('s'),
                                            x: 220,
                                            y: 4
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_OpenFinsButton',

                                            xtype: 'button',
                                            text: dSS._('Open fins'),
                                            width: 131,
                                            scope: this,
                                            handler: this.handleOpen.createDelegate(this),
                                            x: 250,
                                            y: 0
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_CenterFinsButton',

                                            xtype: 'button',
                                            text: dSS._('Center fins'),
                                            width: 131,
                                            handler: this.handleCenter.createDelegate(this),
                                            x: 250,
                                            y: 30
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_CloseFinsButton',

                                            xtype: 'button',
                                            text: dSS._('Close fins'),
                                            width: 131,
                                            handler: this.handleClose.createDelegate(this),
                                            x: 250,
                                            y: 60
                                        },
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            buttons: [
                this.continueButton,
                this.cancelButton
            ]
        });

        dSS.devprops.FinsCalibrationWizard.superclass.constructor.apply(this, arguments);

        this.addEvents('ready2read', 'position1reached', 'position2reached');

        this.on('afterrender', function () {
            this.step2KeyMap = new Ext.KeyMap(this.getId(), [
                {
                    key: Ext.EventObject.DOWN,
                    fn: this.handleClose.createDelegate(this),
                    scope: this,
                    stopEvent: true
                }
            ]);

            this.step2KeyMap.disable();

            this.main.step1.show();
            this.main.step2.hide();

            this.setConfig(         // set INC/ DEC value
                true, 3, 0x3b, this.INC_DEC_VALUE,
                dSS.util.genericRequest.createDelegate(
                    this, [ {
                        url: '/json/device/callScene',      // call scene MAX
                        params: {
                            dsuid: this.dSUID,
                            sceneNumber: 14
                        },
                        scope: this,
                        evt: 'ready2read'
                    } ]
                )
            );
        }, this);

        this.on('ready2read', function (ok) {
            if (this.closed) {
                return;
            }

            if (!ok) {
                this.main.el.mask();
                Ext.Msg.alert(dSS._('Error'), dSS._('Could not call scene on device'));
                return;
            }

            this.readValue(true, 1);
        }, this);

        this.on('position1reached', function () {   // shade reached position 100% (0xffff)
            this.setConfig(                         // now set its position to 50% (2B)
                true, 64, 0x03, this.higherByte(this.SHADE_POSITION_50_PERC),
                this.setConfig.createDelegate(
                    this, [
                        64, 0x02, this.lowerByte(this.SHADE_POSITION_50_PERC),
                        this.setConfig.createDelegate(
                            this, [
                                64, 0x04, 0x00,  // set lamellas to 0 (of 0..255) == Lamellen voll abgekippt
                                this.readValue.createDelegate(this, [ 0 ], 1) // append args at position 1
                            ], 1
                        ) // append args at position 1
                    ], 1
                ) // ^^ append args at position 1
            );
        }, this);

        this.on('position2reached', function () {   // shade reached position 50% (0xffff / 2 == 0x8000)
            this.prepareStep2(true);
        }, this);
    },

    higherByte: function (value) {
        return (value >> 8) & 0x000000ff;
    },
    lowerByte: function (value) {
        return value & 0x000000ff;
    },

    prepareStep2: function (ok) {
        if (ok) {
            this.continueButton.enable();
            this.main.step1.action.incButton.enable();
        }
    },

    handleStop: function () {
        Ext.Ajax.abort(this.tId);
        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
    },

    handleInc: function () {
        this.main.step1.action.incButton.disable();
        this.tempTurnTime += this.loaded.mtimeadjust;
        this.incStepCounter++;
        this.main.step1.action.numsteps.setText(dSS._('Number of steps') + ': ' + this.incStepCounter);

        var scope = this;
        dSS.util.callScene(this.dSUID, dSS.config.scenes.INC, function () {
            scope.main.step1.action.incButton.enable.defer(
                scope.loaded.mtimeadjust * 10 + scope.loaded.reverseTimeOnDirectionChange + 200,
                scope.main.step1.action.incButton
            );
        });
    },

    genericCallback: function (ok) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            this.continueButton.disable();
        } else {
            this.unmask2();
            this.step2KeyMap.enable();
            this.continueButton.enable();
        }
    },

    handleOpen: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var commitOpen = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }
            this.angle = this.finsAngle.OPEN;
            this.setConfig(true, 64, 0x04, 0xff, this.genericCallback.createDelegate(this));
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.continueButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, this.higherByte(this.tempTurnTime),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, this.lowerByte(this.tempTurnTime),
                        commitOpen.createDelegate(this)
                    ], 1
                ) // ^^ append args at position 1
            );
        } else {
            var keepScope = commitOpen.createDelegate(this, [ true ]);
            keepScope();
        }
    },

    handleClose: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var commitClose = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }
            this.angle = this.finsAngle.CLOSED;
            this.setConfig(true, 64, 0x04, 0x00, this.genericCallback.createDelegate(this));
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.continueButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, this.higherByte(this.tempTurnTime),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, this.lowerByte(this.tempTurnTime),
                        commitClose.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        } else {
            var keepScope = commitClose.createDelegate(this, [ true ]);
            keepScope();
        }
    },

    handleCenter: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var commitCenter = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }
            this.angle = this.finsAngle.CENTER;
            this.setConfig(true, 64, 0x04, 0x80, this.genericCallback.createDelegate(this));
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.continueButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, this.higherByte(this.tempTurnTime),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, this.lowerByte(this.tempTurnTime),
                        commitCenter.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        } else {
            var keepScope = commitCenter.createDelegate(this, [ true ]);
            keepScope();
        }
    },

    unmask2: function () {
        this.main.el.unmask();
    },

    handleContinue: function () {
        switch (this.currentStep) {
            case 0:
                this.currentStep++;
                this.main.step2.show();
                this.main.step1.hide();
                this.main.step2.action.turnspinner.setValue(this.tempTurnTime / 100);
                this.setTitle(dSS._('Calibration of fins turn time: step 2 of 2'));
                this.continueButton.setText(dSS._('Finish'));
                this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
                this.setConfig(true, 64, 0x04, 0x00, this.delayStepBetween1And2.createDelegate(this));     // set lamellas to 0 (of 0..255) == Lamellen voll abgekippt
                break;
            case 1:
                this.continueButton.disable();
                this.cancelButton.disable();
                this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');

                var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);
                this.tempTurnTime = t;

                this.setConfig(
                    true, 3, 0x3b, this.loaded.mtimeadjust,                                      // 1. Restore original INC/DEC value
                    this.setConfig.createDelegate(                                               // 2. Set turn time calibration status to 0x6 (manually calibrated)
                        this, [
                            1, 0x5c, 0b01100000 + this.motionTimeAndDirectionCalibrationStatus, this.setConfig.createDelegate( // 3. Write blades turn time (0x3e, 0x3f)
                                this, [
                                    3, 0x3f, this.higherByte(this.tempTurnTime), this.setConfig.createDelegate(
                                        this, [
                                            3, 0x3e, this.lowerByte(this.tempTurnTime), this.setConfig.createDelegate(      // 4. Change MODE to calibrated
                                                this, [ 3, 0x00, 33, this.finishWizard.createDelegate(this) ], 1
                                            )
                                        ], 1
                                    )
                                ], 1
                            )
                        ], 1
                    )
                );
                break;
            default:
                this.closed = true;
                this.close();
                break;
        }
    },

    finishWizard: function (ok) {
        if (ok) {
            this.closeWizard();
        } else {
            var keepScope = this.closeWizard.createDelegate(this);
            keepScope();
        }
    },

    cancelWizard: function () {
        this.continueButton.disable();
        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.setConfig(
            true, 3, 0x3b, this.loaded.mtimeadjust, this.closeWizard.createDelegate(this)     // 1. Restore original INC/DEC value
        );
    },

    closeWizard: function () {
        this.closed = true;
        this.close();
    },

    setConfig: function (ok, cls, index, value, callback) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.tId = dSS.util.genericRequest({
            url: '/json/device/setConfig',
            params: {
                class: cls,
                index: index,
                dsuid: this.dSUID,
                value: value
            },
            failureMsg: dSS._('Could not write device configuration'),
            scope: this
        }, callback);
    },

    // direction 0: read until position <= this.SHADE_POSITION_50_PERC
    // direction 1: read until position >= this.SHADE_POSITION_100_PERC
    // direction 2: read position and execute callback(value)
    readValue: function (ok, direction, callback) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.tId = Ext.Ajax.request({           // read current position value (0..0xffff)
            url: '/json/device/getConfigWord',
            method: 'GET',
            params: {
                class: 64,
                index: 0x06,
                dsuid: this.dSUID
            },
            scope: this,
            success: function (response) {
                if (this.closed) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (!jsonData || !jsonData.ok) {
                        Ext.Msg.alert(
                            dSS._('Error'),
                            dSS._('Could not load current fin position'), ': ' + dSS._(jsonData.message)
                        );
                        this.unmask2();
                        this.main.el.mask();
                        return;
                    }

                    if (!isNaN(jsonData.result.value)) {
                        var value = jsonData.result.value; // + 32767; // int -> uint PROBABLY NOT NEEDED NOW
                        if (direction === 1) {
                            if (value >= this.SHADE_POSITION_100_PERC) {
                                this.fireEvent('position1reached');
                                return;
                            }
                            this.lastReadValue = value;
                        } else if (direction === 0) {
                            if (value <= this.SHADE_POSITION_50_PERC + 0.1 * this.SHADE_POSITION_100_PERC && this.lastReadValue === value) {
                                this.fireEvent('position2reached');
                                this.tempStartPos = value;
                                return;
                            }
                            this.lastReadValue = value;
                        } else if (callback) {
                            callback(value);
                            return;
                        }
                        this.readValue.defer(500, this, [ true, direction ]);
                    } else {
                        throw dSS._('received invalid value');
                    }
                } catch (err) {
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not load current fin position') + ': ' + err
                    );
                    this.main.el.mask();
                }
            },
            failure: function (response) {
                if (this.closed) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(
                        response.responseText
                    );
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not load current fin position') + ': ' + dSS._(jsonData.message)
                    );
                } catch (err) {
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not load current fin position') + ': ' + err
                    );
                }
                this.main.el.mask();
            }
        });
    },

    delayStepBetween1And2: function (ok) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.configStepBetween1And2.defer(this.tempTurnTime + 300, this);
    },

    configStepBetween1And2: function () {
        if (this.closed) {
            return;
        }

        var newPos = this.tempStartPos - 0x0800;
        if (newPos < 0) {
            newPos = 0;
        }

        this.setConfig(
            true, 64, 0x03, this.higherByte(newPos),     // set shade position
            this.setConfig.createDelegate(this, [
                64, 0x02, this.lowerByte(newPos),
                this.setConfig.createDelegate(this, [
                    3, 0x3f, this.higherByte(this.tempTurnTime),    // turn time adjustment
                    this.setConfig.createDelegate(this, [
                        3, 0x3e, this.lowerByte(this.tempTurnTime),
                        this.configStepBetween1And2Done.createDelegate(this)
                    ], 1) // append args at position 1
                ], 1) // append args at position 1
            ], 1) // append args at position 1
        );
    },

    configStepBetween1And2Done: function (ok) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.main.step2.action.turnspinner.setValue(this.tempTurnTime / 100);
        this.step2KeyMap.enable();
        this.unmask2();
    }
});

Ext.reg('dssdevfinscalibrationwizard', dSS.devprops.FinsCalibrationWizard);
